@@ -1,7 +1,7 @@
 ################################################################################
 ## package 'openCR'
 ## openCR.esa.R
-## 2011-12-30, 2013-01-20
+## 2019-05-17
 ################################################################################
 
 
@@ -17,6 +17,9 @@
 # JSSAsecrg = 24
 # JSSAsecrgCL = 25
 
+# secrCL = 30
+# secrD = 31
+
 openCR.esa <- function (object, bysession = FALSE) {
 
     if (!(grepl('secr',object$type) & inherits(object,'openCR')) )
@@ -43,15 +46,14 @@ openCR.esa <- function (object, bysession = FALSE) {
     ###### movement kernel and related
     cellsize <- mqarray <- 0
     kernel <- data.frame(rownames=NULL)
-    if (movementmodel %in% c('normal','exponential','user')) {
+    if (movementmodel %in% c('normal','exponential','user','t2D')) {
         k2 <- details$kernelradius
         cellsize <- attr(mask,'area')^0.5 * 100   ## metres, equal mask cellsize
         kernel <- expand.grid(x = -k2:k2, y = -k2:k2)
         kernel <- kernel[(kernel$x^2 + kernel$y^2) <= (k2+0.5)^2, ]
         mqarray <- mqsetup (mask, kernel, cellsize)
     }
-    movemodel <- switch (movementmodel, static = 0, uncorrelated = 1, normal = 2, 
-                         exponential = 3, user = 4) 
+    movemodel <- movecode(movementmodel)
 
     #--------------------------------------------------------------------
     # Fixed beta
@@ -67,10 +69,7 @@ openCR.esa <- function (object, bysession = FALSE) {
     nc  <- nrow(capthist)
     J <- length(intervals) + 1
 
-    type <- switch(type, CJSsecr = 6, JSSAsecrfCL = 9, JSSAsecrlCL = 10, JSSAsecrbCL = 11, JSSAsecrgCL = 25,
-                   JSSAsecrf = 7, JSSAsecrl = 12, JSSAsecrb = 13, JSSAsecrB = 14, JSSAsecrg = 24,
-                   JSSAsecrD = 8)
-
+    type <- typecode(type)  # convert to integer
     trps <- traps(capthist)
     k <- nrow(trps)
     m <- nrow(mask)
@@ -78,29 +77,22 @@ openCR.esa <- function (object, bysession = FALSE) {
     else area <- 0
     distrib <- switch (object$distribution, poisson = 0, binomial = 1)
 
-    usge <- usage(traps(capthist))
+    detectr <- detector(trps)[1]
+    if (detectr == 'count') {
+        detectr <- if (object$binomN == 0) "poissoncount" else "binomialcount"
+    }
+
+    usge <- usage(trps)
     if (is.null(usge)) usge <- matrix(1, nrow = k, ncol = ncol(capthist))
 
-    if (object$ncores>1)
-        temp <- makegkParallelcpp (as.integer(object$detectfn), 
+    temp <- makegkParallelcpp (as.integer(object$detectfn), 
                                as.integer(.openCRstuff$sigmai[type]),
                                as.integer(details$grain),
                                as.matrix(realparval0),
                                as.matrix(trps),
                                as.matrix(object$mask))
-    else
-        temp <- makegkcpp(
-            as.integer(nrow(realparval0)),     # number of rows in lookup table
-            as.integer(k),                     # detectors
-            as.integer(m),                     # mask points
-            as.integer(object$detectfn),
-            as.integer(.openCRstuff$sigmai[type]),                # column index of sigma in realparval
-            as.double(realparval0),
-            as.double(unlist(trps)),
-            as.double(unlist(mask)))
-        
-    
     gk0 <- temp[[1]]
+    hk0 <- temp[[2]]
 
     ## mixture proportions
     if (details$nmix > 1) {
@@ -112,52 +104,28 @@ openCR.esa <- function (object, bysession = FALSE) {
         pmix <- matrix(1, nrow = details$nmix, ncol = nc)
     }
     onea <- function (x) {
-        if (object$ncores == 1) {
-            pch1 <-  PCH1secrcpp(
-                as.integer(type),
-                as.integer(x-1),
-                as.integer(nc),
-                as.integer(J),
-                as.integer(cumss),
-                as.integer(k),
-                as.integer(m),
-                as.matrix(realparval0),
-                as.integer(object$design0$PIA),
-                as.integer(object$design$PIAJ),
-                as.double (gk0),
-                as.integer(binomN),
-                as.matrix (usge),
-                as.double (intervals),
-                as.integer(object$moveargsi),
-                as.integer(movemodel),
-                as.character(object$usermodel),
-                as.matrix(kernel),
-                as.matrix(mqarray),
-                as.double(cellsize))
-        }
-        else {
-            pch1 <-  PCH1secrparallelcpp(
-                as.integer(x-1),
-                as.integer(type),
-                as.integer(object$details$grain),
-                as.logical(individual),
-                as.integer(J),
-                as.integer(m),
-                as.integer(nc),
-                as.integer(cumss),
-                as.matrix (realparval0),
-                as.integer(object$design0$PIA),
-                as.integer(object$design0$PIAJ),
-                as.double (gk0),
-                as.integer(binomN),
-                as.matrix (usge),
-                as.double (intervals),
-                as.integer(object$moveargsi),
-                as.integer(movemodel),
-                as.matrix(kernel),
-                as.matrix(mqarray),
-                as.double (cellsize))
-        }
+        pch1 <-  PCH1secrparallelcpp(
+            as.integer(x-1),
+            as.integer(type),
+            as.integer(object$details$grain),
+            as.logical(individual),
+            as.integer(J),
+            as.integer(m),
+            as.integer(nc),
+            as.integer(cumss),
+            as.matrix (realparval0),
+            as.integer(object$design0$PIA),
+            as.integer(object$design0$PIAJ),
+            as.double (if (detectr == "poissoncount") hk0 else gk0),  ## 2019-05-08, 17
+            as.integer(binomN),
+            as.matrix (usge),
+            as.double (intervals),
+            as.integer(object$moveargsi),
+            as.integer(movemodel),
+            as.character(object$usermodel),
+            as.matrix(kernel),
+            as.matrix(mqarray),
+            as.double (cellsize))
         pmix[x,] * pch1
     }
     
